#################
## Code for use with      ##
## Gary Koop's book      ##
## Analysis of                ##
## Economic data 2Ed   ##
##                                ##
## K Shutes                  ##
## 2008-03-10 10:19   ##
##                                 ##
#################

require(urca)
require(vars)
require(lmtest)

## Order of models not specified to allow student selection

## Ex 11.1

NYSE<-read.table('../Text/NYSE.txt',header=T)      
attach(NYSE)

NYSE<-ts(NYSE,freq=12,start=c(1952,1))
plot(NYSE[,2])
acf(Return)
pacf(Return)

arima(NYSE[,2])



## Doing the same inelegantly

lagdd<-embed(Return,3)
colnames(lagdd)<-c('R_t2','R_t1','R_t')
lagdd<-data.frame(lagdd)
attach(lagdd)

fit<-lm(R_t~R_t1+R_t2)
summary(fit)

dy<-Return-mean(Return)

dy2<-dy^2

plot(dy2,type='l',ylab='Vol')

dyy<-(NYSE[,2]-mean(NYSE[,2]))^2
plot(dyy,ylab='Vol')


## Can use dyy instead makes no difference

arima(dy2,order=c(3,0,0))

arima(dy2,order=c(2,0,0))

arima(dy2,order=c(1,0,0))

## Doing the same inelegantly
voldd<-embed(dy2,3)
colnames(voldd)<-c('Vol_t2','Vol_t1','Vol_t')
voldd<-data.frame(voldd)
attach(voldd)

fit1<-lm(Vol_t~Vol_t1+Vol_t2)
fit2<-update(fit1,.~.-Vol_t2)



## Ex 11.2
wp<-read.table('../Text/WP.txt',header=T)      
attach(wp)
wp2<-wp[,2:3]
attach(wp2)
wp2<-ts(wp2,freq=1,start=c(1855))
wp3<-diff(wp2)
colnames(wp3)<-c('DlogW','DlogP')
Fit<-VAR(wp3,p=4,'both')
summary(Fit)


## Ex 11.3
long<-read.table('../Text/LONGGDP.txt',header=T)      
attach(long)

long<-ts(long[,-1],freq=1,start=c(1870))

plot(long,main='Time Series of Real GDP')
dlong<-diff(long)
colnames(dlong)<-c('dUS','dUK','dCAN','dOZ')

fit11<-VAR(dlong,p=4,'both')
summary(fit11)


grangertest(dlong[,1],dlong[,2],order=4)
grangertest(dlong[,2],dlong[,1],order=4)
grangertest(dlong[,1],dlong[,3],order=4)
grangertest(dlong[,3],dlong[,1],order=4)
grangertest(dlong[,1],dlong[,4],order=4)
grangertest(dlong[,4],dlong[,1],order=4)

grangertest(dlong[,2],dlong[,3],order=4)
grangertest(dlong[,3],dlong[,2],order=4)
grangertest(dlong[,2],dlong[,4],order=4)
grangertest(dlong[,4],dlong[,2],order=4)



## Ex 11.4
inc<-read.table('../Text/INCOME.txt',header=T)      
attach(inc)

fitres<-lm(Consumption~Income)$resid
summary(ur.df(fitres,'none'))



grangertest(Consumption,Income,order=4)
grangertest(Income,Consumption,order=4)
grangertest(Consumption,Income,order=3)
grangertest(Income,Consumption,order=3)
grangertest(Consumption,Income,order=2)
grangertest(Income,Consumption,order=2)
grangertest(Consumption,Income,order=1)
grangertest(Income,Consumption,order=1)


## Ex 11.5
rmpy<-read.table('../Text/RMPY.txt',header=T)      
attach(rmpy)

summary(rmpy)
summary(ur.df(R,'drift'))
summary(ur.df(M,'drift'))
summary(ur.df(P,'drift'))
summary(ur.df(Y,'drift'))

fitrmres<-lm(R~M)$resid
fitrpres<-lm(R~P)$resid
fitryres<-lm(R~Y)$resid
fitmpres<-lm(M~P)$resid
fitmyres<-lm(M~Y)$resid
fitpyres<-lm(P~Y)$resid

fitres<-cbind(fitrmres,fitrpres,fitryres,fitmpres,fitmyres,fitpyres)
i=1:6



## Not pretty! 
for(j in i)
{
    print(summary(ur.df(fitres[,j],lags=4,'none')))
    print(summary(ur.df(fitres[,j],lags=3,'none')))    
    print(summary(ur.df(fitres[,j],lags=2,'none')))
    print(summary(ur.df(fitres[,j],lags=1,'none')))
    }

subdata<-na.omit(rmpy)
detach(rmpy);attach(subdata)
summary(ur.df(DR,'drift'))
summary(ur.df(DM,'drift'))
summary(ur.df(DP,'drift'))
summary(ur.df(DY,'drift'))


## Ex 11.7
i=5:1
VARs<-lapply(i,VAR,y=cbind(DR,DM,DP,DY),type='both')


## Ex 11.8

long<-read.table('../Text/LONGGDP.txt',header=T)      
attach(long)

long<-ts(long[,-1],freq=1,start=c(1870))

dlong<-diff(long)


VARselect(dlong,lag.max=5,'both')
VARlong1<-VAR(dlong,p=1,'both')
VARlong2<-VAR(dlong,p=2,'both')
VARlong3<-VAR(dlong,p=3,'both')
VARlong4<-VAR(dlong,p=4,'both')
summary(VARlong1)
summary(VARlong2)
summary(VARlong3)
summary(VARlong4)

##Ex 11.9
rmpy<-read.table('../Text/RMPY.txt',header=T)      
attach(rmpy)
drmpy<-cbind(DY,DP,DR,DM)
drmpy<-na.omit(drmpy)
drmpy<-ts(drmpy,freq=4,start=c(1947,1))
drmpy<-window(drmpy,end=c(1991,4))
VARrmpy1<-VAR(drmpy,p=1,'both')
VARrmpy2<-VAR(drmpy,p=2,'both')
VARrmpy3<-VAR(drmpy,p=3,'both')
VARrmpy4<-VAR(drmpy,p=4,'both')
summary(VARrmpy1)
summary(VARrmpy2)
summary(VARrmpy3)
summary(VARrmpy4)

predict(VARrmpy1,n.ahead=4,ci=0.95)
predict(VARrmpy2,n.ahead=4,ci=0.95)
predict(VARrmpy3,n.ahead=4,ci=0.95)
predict(VARrmpy4,n.ahead=4,ci=0.95)


drmpy<-window(drmpy,end=c(1990,4))
VARrmpy1<-VAR(drmpy,p=1,'both')
VARrmpy2<-VAR(drmpy,p=2,'both')
VARrmpy3<-VAR(drmpy,p=3,'both')
VARrmpy4<-VAR(drmpy,p=4,'both')
summary(VARrmpy1)
summary(VARrmpy2)
summary(VARrmpy3)
summary(VARrmpy4)

predict(VARrmpy1,n.ahead=8,ci=0.95)
predict(VARrmpy2,n.ahead=8,ci=0.95)
predict(VARrmpy3,n.ahead=8,ci=0.95)
predict(VARrmpy4,n.ahead=8,ci=0.95)



drmpy<-window(drmpy,end=c(1989,4))
VARrmpy1<-VAR(drmpy,p=1,'both')
VARrmpy2<-VAR(drmpy,p=2,'both')
VARrmpy3<-VAR(drmpy,p=3,'both')
VARrmpy4<-VAR(drmpy,p=4,'both')
summary(VARrmpy1)
summary(VARrmpy2)
summary(VARrmpy3)
summary(VARrmpy4)

predict(VARrmpy1,n.ahead=12,ci=0.95)
predict(VARrmpy2,n.ahead=12,ci=0.95)
predict(VARrmpy3,n.ahead=12,ci=0.95)
predict(VARrmpy4,n.ahead=12,ci=0.95)


drmpy<-window(drmpy,end=c(1988,4))
VARrmpy1<-VAR(drmpy,p=1,'both')
VARrmpy2<-VAR(drmpy,p=2,'both')
VARrmpy3<-VAR(drmpy,p=3,'both')
VARrmpy4<-VAR(drmpy,p=4,'both')
summary(VARrmpy1)
summary(VARrmpy2)
summary(VARrmpy3)
summary(VARrmpy4)

predict(VARrmpy1,n.ahead=16,ci=0.95)
predict(VARrmpy2,n.ahead=16,ci=0.95)
predict(VARrmpy3,n.ahead=16,ci=0.95)
predict(VARrmpy4,n.ahead=16,ci=0.95)


## Ex 11.10

long<-read.table('../Text/LONGGDP.txt',header=T)      
attach(long)

long<-ts(long[,-1],freq=1,start=c(1870))

dlong<-diff(long)
dlong2<-window(dlong,end=1981)
## Select order of VAR
ord<-min(VARselect(dlong2,lag.max=8,'both')$selection)

predict(VAR(dlong,p=ord,'both'),n.ahead=12)

dlong2<-window(dlong,end=1984)
## Select order of VAR
ord<-min(VARselect(dlong2,lag.max=8,'both')$selection)
pred<-predict(VAR(dlong,p=ord,'both'),n.ahead=9)
win<-window(dlong,start=1985,end=1993)
colnames(win)<-c('US','UK','CAN','OZ')
ts.union(pred$fcst$US,win[,1])
ts.union(pred$fcst$UK,win[,2])
ts.union(pred$fcst$CAN,win[,3])
ts.union(pred$fcst$OZ,win[,4])


##Ex11.11
orange<-read.table('../Text/ORANGE.txt',header=T)      
attach(orange)
ora1<-ca.jo(orange,ecdet='const',K=2)
ora.vecm<-cajorls(ora1,1)
summary(ora.vecm$rlm)

ora2<-ca.jo(orange,ecdet='const',K=3)
ora2.vecm<-cajorls(ora2,1)
summary(ora2.vecm$rlm)

ora3<-ca.jo(orange,ecdet='const',K=4)
ora3.vecm<-cajorls(ora3,1)
summary(ora3.vecm$rlm)

ora4<-ca.jo(orange,ecdet='const',K=5)
ora4.vecm<-cajorls(ora4,1)
summary(ora4.vecm$rlm)

## Select or.vecm as example
orangets<-ts(orange,freq=12,start=c(1980,1))
orts2<-window(orangets,start=c(1980,1),end=c(1990,12))
vora<-ca.jo(orts2,ecdet='const',K=2)
vora.vecm<-cajorls(vora,1)
summary(vora.vecm$rlm)

varor.vecm<-vec2var(vora,r=1)
preo<-predict(varor.vecm,n.ahead=49)
orangets2<-window(orangets,start=c(1991,1))
ts.union(preo$fcst$regular,orangets2[,2])
ts.union(preo$fcst$organic,orangets2[,1])

orts2<-window(orangets,start=c(1980,1),end=c(1992,12))
vora<-ca.jo(orts2,ecdet='const',K=2)
vora.vecm<-cajorls(vora,1)
summary(vora.vecm$rlm)

varor.vecm<-vec2var(vora,r=1)
preo<-predict(varor.vecm,n.ahead=25)
orangets2<-window(orangets,start=c(1993,1))
ts.union(preo$fcst$regular,orangets2[,2])
ts.union(preo$fcst$organic,orangets2[,1])



## Ex 11.12
inc<-read.table('../Text/INCOME.txt',header=T)      
attach(inc)
inc<-inc[,-1]
inc1<-ca.jo(inc,ecdet='const',K=2)
inc.vecm<-cajorls(inc1,1)
summary(inc.vecm$rlm)

inc2<-ca.jo(inc,ecdet='const',K=3)
inc2.vecm<-cajorls(inc2,1)
summary(inc2.vecm$rlm)

inc3<-ca.jo(inc,ecdet='const',K=4)
inc3.vecm<-cajorls(inc3,1)
summary(inc3.vecm$rlm)

inc4<-ca.jo(inc,ecdet='const',K=5)
inc4.vecm<-cajorls(inc4,1)
summary(inc4.vecm$rlm)

incomets<-ts(inc,freq=4,start=c(1954,1))

dincomets<-diff(incomets)
ord<-min(VARselect(dincomets)$selection)
VAR(dincomets,p=ord,'trend')

wdin<-window(dincomets,end=c(1990,4))
ord<-min(VARselect(wdin)$selection)
vpred<-VAR(wdin,p=ord,'trend')
pred<-predict(vpred,n.ahead=16)
wend<-window(dincomets,start=c(1991,1))
ts.union(pred$fcst$Income,wend[,1])
ts.union(pred$fcst$Consumption,wend[,2])
