#################
## Code for use with      ##
## Gary Koop's book      ##
## Analysis of                ##
## Economic data 2Ed   ##
##                                ##
## K Shutes                  ##
## 2008-03-10 10:19   ##
##                                 ##
#################


## Creates new plots automatically for windows, mac and Linux
newgraph<-function(platform=.Platform$GUI)
{
    print(platform)
    if(platform=='Rgui')
    return(windows())
    if(platform=='AQUA')
    return(quartz())
    else
    return(x11())
    }
   
 pause<-function()
 {
    cat('Paused. Pres <Enter> to continue')
    readline()
    invisible()
    
    
    }       

## Chapter 2: Exercise 2.1 Graphical Methods

EXRUK<-read.table('../Text/EXRUK.txt',header=F)                     ## Creates Figure 2.1
EXRUK<-ts(EXRUK,frequency=12,start=c(1947,1))
EXRUK<-EXRUK[,-1]
newgraph()
plot(EXRUK,ylab='Pence per dollar')
pause()


income<-read.table('../Text/Income.txt',header=T)                   ##Reads in data (log Incomes & Consumption)
income<-ts(income,frequency=4,start=c(1954,1))                   ## Makes in to quarterly Time Series
income<-income[,-1]                                                               ## Removes the false date column
growth<-100*(income-lag(income,-1)   )                                  ## Create Growth rates

newgraph()
plot(growth[,1],ylab='Gowth in Personal Income')                     ## Creates a plot of the growth in incomes
pause()

## Exercise 2.2 Histogram

GDP<-read.table('../Text/GDPPC.txt',header=F)
GDP<-GDP[,-1]
colnames(GDP)<-c('Country','Year','GDP')

newgraph()

## There a slight differences here as there are different bins

par(mfrow=c(3,1))
breakh=c(10,5,15)
hist(GDP$GDP,freq=T,breaks=10,xlab='GDP',col='skyblue',main=paste('Histogram of GDP Distribution with ',breakh[[1]],' breaks'))
hist(GDP$GDP,freq=T,breaks=5,xlab='GDP',col='red',main=paste('Histogram of GDP Distribution with ',breakh[[2]],' breaks'))
hist(GDP$GDP,freq=T,breaks=15,xlab='GDP',col='green',main=paste('Histogram of GDP Distribution with ',breakh[[3]],' breaks'))
pause()


## Exercise 2.3 Scatterplots

forest<-read.table('../Text/Forest1.txt',header=T)                  ## Note that the file is different as the headers are spread over 2 lines in the original
attach(forest)
par(mfrow=c(3,1))
plot(Popdensity,Forestloss,xlim=c(0,3000),ylim=c(0,6),ylab='Average Annual Forest Loss (%)',xlab='Population per 1000 ha',pch=23,col='red',bg='red')
plot(Cropchange,Forestloss,ylim=c(0,6),ylab='Average Annual Forest Loss (%)',xlab='Percentage increase in Crop land (%)',pch=23,col='blue',bg='blue')
plot(Pasturechange,Forestloss,ylim=c(0,6),ylab='Average Annual Forest Loss (%)',xlab='Percentage increase in Pasture (%)',pch=23,col='green',bg='green')
pause()
## Exercise 2.4 summary statistics

print(mean(forest))
print(sd(forest))
print(summary(forest))

## Data for Indices
year<-seq(1999,2003,by=1)
bananaprice=c(0.89,.91,.91,.94,.95)
appleprice=c(0.44,0.43,.46,.5,.51)
kiwiprice=c(1.58,1.66,1.9,2.1,2.25)

bananaq<-c(100,100,98,94,96)
appleq<-c(78,82,86,87,88)
kiwiq<-c(1,1,3,4,5)


bananas<-cbind(year,bananaprice)
apples<-cbind(year,appleprice)
kiwis<-cbind(year,appleprice)


## Creating an Index

index<-function(data,base=100,baseyear=1990,dp=2)                ## Use data<-cbind(year,price)
{
    basedivisor<-subset(data[,2],data[,1]==baseyear)
    dataI<-round(data[,2]/basedivisor*base,dp)
    new<-cbind(data[,1],dataI)
    colnames(new)=c('Date','Index')
    index=ts(dataI,start=data[1,1])
    }



print(index(bananas,base=100,baseyear=2000))

## Laspeyres Index
laspeyres<-function(qdata,pdata,year,baseyear=2000)
{
    tdata<-cbind(year,qdata,pdata)
    qbase<-subset(tdata[,2:(1+dim(qdata)[2])],tdata[,1]==baseyear)
    a<-seq(1,nrow(qdata))
    numerator<-sapply(a,function(a){sum(qbase*pdata[a,])})
    denominator<-sum(qbase*subset(tdata[,(2+dim(qdata)[2]):dim(tdata)[2]],tdata[,1]==baseyear))
    LI=100*numerator/denominator
    LI=ts(LI,start=year[1])
    list(LPI=LI)
    }
    
 prices<-cbind(bananaprice,appleprice,kiwiprice)
 quantities<-cbind(bananaq,appleq,kiwiq)
 l=laspeyres(quantities,prices,year,baseyear=2000)
 print(l)
 
 paasche<-function(qdata,pdata,year,baseyear=2000)
 {
      tdata<-cbind(year,qdata,pdata)
      pbase<-subset(tdata[,(2+dim(qdata)[2]):dim(tdata)[2]],tdata[,1]==baseyear)
      a<-seq(1,nrow(qdata))
      denominator<-sapply(a,function(a){sum(pbase*qdata[a,])})
      numerator<-rowSums(pdata*qdata)
      PI<-100*numerator/denominator
      PI<-ts(PI,start=year[1])
     list(PPI=PI)
    }